Documentation for the examples
Each file is a standalone executable with its own main function. Examples vary from basic hello world simplicity to some more complex examples. They may contain logic bugs, however their purpose is for demonstration and for testing UI elements. If you would like to contribute fixes, improvements or more examples, you’re very welcome to do so.
More interesting examples can be found in the fltk-rs-demos repo. Also a nice implementation of the 7guis tasks can be found here. Various advanced examples can also be found here.
Modules
- Documentation for the
animationsexample - Documentation for the
calculatorexample - Documentation for the
calculator2example - Documentation for the
chartsexample - Documentation for the
closable_tabexample - Documentation for the
composite_widgetsexample - A simple counter.
- A counter using messaging over a channel.
- A a flutter-styled incrementing counter.
- A counter using event handling.
- Documentation for the
custom_choiceexample - Documentation for the
custom_dialexample - Documentation for the
custom_popupexample - Documentation for the
defaultsexample - Documentation for the
editorexample - Documentation for the
editor2example - Documentation for the
fbexample - Documentation for the
flexexample - Documentation for the
format_textexample - Documentation for the
framesexample - Documentation for the
gradientsexample - Documentation for the
hello_buttonexample - Documentation for the
hello_flexexample - Documentation for the
hello_svgexample - Documentation for the
imageexample - Documentation for the
menubuttonexample - Documentation for the
messagesexample - Documentation for the
paintexample - Documentation for the
pongexample - Documentation for the
popup_browserexample - Documentation for the
prefsexample - Documentation for the
rgbexample - Documentation for the
rounded_imagesexample - Documentation for the
screens_infoexample - Documentation for the
shapedwindow_taskbarexample - Documentation for the
spreadsheetexample - Documentation for the
system_fontsexample - Documentation for the
tableexample - Documentation for the
tabsexample - Documentation for the
temp_converterexample - How to show a window from another thread.
- Documentation for the
tileexample - Documentation for the
treeexample - Documentation for the
widget_tableexample - Documentation for the
wizardexample