Struct fltk::window::GlutWindow

source ·
pub struct GlutWindow { /* private fields */ }
Creates a OpenGL Glut window widget

Implementations

Creates a default initialized glut window

Gets an opengl function address

Forces the window to be drawn, this window is also made current and calls draw()

Returns whether the OpeGL context is still valid

Mark the OpeGL context as still valid

Returns whether the context is valid upon creation

Mark the context as valid upon creation

Returns the GlContext

Sets the GlContext

Swaps the back and front buffers

Sets the projection so 0,0 is in the lower left of the window and each pixel is 1 unit wide/tall.

Returns whether the GlutWindow can do overlay

Redraws the overlay

Hides the overlay

Makes the overlay current

Returns the pixels per unit/point

Gets the window’s width in pixels

Gets the window’s height in pixels

Get the Mode of the GlutWindow

Set the Mode of the GlutWindow