Struct fltk::window::SingleWindow
source · pub struct SingleWindow { /* private fields */ }Expand description
Creates a single (buffered) window widget
Implementations
sourceimpl SingleWindow
impl SingleWindow
sourcepub fn default() -> SingleWindow
pub fn default() -> SingleWindow
Creates a default initialized single window
Note: Only call this from the main thread.
sourcepub unsafe fn find_by_handle(handle: RawHandle) -> Option<impl WindowExt>
pub unsafe fn find_by_handle(handle: RawHandle) -> Option<impl WindowExt>
Find an Fl_Window through a raw handle. The window must have been instantiated by the app.
void * to: (Windows: HWND, X11: Xid (u64), macOS: NSWindow)
Safety
The data must be valid and is OS-dependent.
sourcepub fn show_with_env_args(&mut self)
pub fn show_with_env_args(&mut self)
Use FLTK specific arguments for the application: More info here. The options are:
-bg2 color-bg color-di[splay] host:n.n-dn[d]-fg color-g[eometry] WxH+X+Y-i[conic]-k[bd]-na[me] classname-nod[nd]-nok[bd]-not[ooltips]-s[cheme] scheme-ti[tle] windowtitle-to[oltips]
sourcepub fn show_with_args(&mut self, args: &[&str])
pub fn show_with_args(&mut self, args: &[&str])
Use FLTK specific arguments for the application: More info here. The options are:
-bg2 color-bg color-di[splay] host:n.n-dn[d]-fg color-g[eometry] WxH+X+Y-i[conic]-k[bd]-na[me] classname-nod[nd]-nok[bd]-not[ooltips]-s[cheme] scheme-ti[tle] windowtitle-to[oltips]
sourcepub fn pixels_per_unit(&self) -> f32
pub fn pixels_per_unit(&self) -> f32
Returns the pixels per unit/point
sourcepub fn default_xclass() -> Option<String>
pub fn default_xclass() -> Option<String>
Get the default XA_WM_CLASS property for all windows of your application
sourcepub fn set_default_xclass(s: &str)
pub fn set_default_xclass(s: &str)
Set the default XA_WM_CLASS property for all windows of your application. This should be called before showing with window
sourcepub fn set_on_top(&self)
pub fn set_on_top(&self)
Set the borderless window to be on top of the macos system menu bar