pub struct Color { /* private fields */ }Expand description
Defines colors used by FLTK.
Colors are stored as RGBI values, the last being the index for FLTK colors in this enum.
Colors in this enum don’t have an RGB stored. However, custom colors have an RGB, and don’t have an index.
The RGBI can be acquired by casting the color to u32 and formatting it to 0x{08x}.
The last 2 digits are the hexadecimal representation of the color in this enum.
For example, Color::White, has a hex of 0x000000ff, ff being the 255 value of this enum.
A custom color like Color::from_u32(0x646464), will have an representation as 0x64646400,
of which the final 00 indicates that it is not stored in this enum.
For convenience, the fmt::Display trait is implemented so that the name of the Color is shown
when there is one, otherwise the RGB value is given.
Implementations
sourceimpl Color
impl Color
sourcepub const ForeGround: Color = _
pub const ForeGround: Color = _
ForeGround, label colors
sourcepub const Foreground: Color = _
pub const Foreground: Color = _
Foreground, label colors
sourcepub const BackGround2: Color = _
pub const BackGround2: Color = _
BackGround2, Is the color inside input, output and text display widgets
sourcepub const Background2: Color = _
pub const Background2: Color = _
Background2, Is the color inside input, output and text display widgets
sourcepub const FrameDefault: Color = _
pub const FrameDefault: Color = _
FrameDefault
sourcepub const BackGround: Color = _
pub const BackGround: Color = _
BackGround
sourcepub const Background: Color = _
pub const Background: Color = _
Background
sourcepub const DarkYellow: Color = _
pub const DarkYellow: Color = _
DarkYellow
sourcepub const DarkMagenta: Color = _
pub const DarkMagenta: Color = _
DarkMagenta
sourcepub const XtermBlack: Color = _
pub const XtermBlack: Color = _
ANSI/xterm Black
sourcepub const XtermGreen: Color = _
pub const XtermGreen: Color = _
ANSI/xterm Green
sourcepub const XtermYellow: Color = _
pub const XtermYellow: Color = _
ANSI/xterm Yellow
sourcepub const XtermMagenta: Color = _
pub const XtermMagenta: Color = _
ANSI/xterm Magenta
sourcepub const XtermWhite: Color = _
pub const XtermWhite: Color = _
ANSI/xterm White
sourcepub const XtermBgRed: Color = _
pub const XtermBgRed: Color = _
ANSI/xterm background Red
sourcepub const XtermBgGreen: Color = _
pub const XtermBgGreen: Color = _
ANSI/xterm background Green
sourcepub const XtermBgYellow: Color = _
pub const XtermBgYellow: Color = _
ANSI/xterm background Yelllow
sourcepub const XtermBgBlue: Color = _
pub const XtermBgBlue: Color = _
ANSI/xterm background Blue
sourcepub const XtermBgMagenta: Color = _
pub const XtermBgMagenta: Color = _
ANSI/xterm background Magenta
sourcepub const XtermBgCyan: Color = _
pub const XtermBgCyan: Color = _
ANSI/xterm background Cyan
sourcepub const XtermBgWhite: Color = _
pub const XtermBgWhite: Color = _
ANSI/xterm background White
sourcepub const TransparentBg: Color = _
pub const TransparentBg: Color = _
Special background color value that lets the Terminal widget’s box() color show through behind the text.
sourcepub const fn from_rgb(r: u8, g: u8, b: u8) -> Color
pub const fn from_rgb(r: u8, g: u8, b: u8) -> Color
Returns a color from RGB
Examples found in repository?
3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103
fn main() {
let a = app::App::default().with_scheme(app::Scheme::Gtk);
let mut win = window::Window::default().with_size(640, 480);
let mut col = group::Flex::default_fill().column();
main_panel(&mut col);
col.end();
win.resizable(&col);
win.set_color(enums::Color::from_rgb(250, 250, 250));
win.end();
win.show();
win.size_range(600, 400, 0, 0);
a.run().unwrap();
}
fn buttons_panel(parent: &mut group::Flex) {
frame::Frame::default();
let w = frame::Frame::default().with_label("Welcome to Flex Login");
let mut urow = group::Flex::default().row();
{
frame::Frame::default()
.with_label("Username:")
.with_align(enums::Align::Inside | enums::Align::Right);
let username = input::Input::default();
urow.fixed(&username, 180);
urow.end();
}
let mut prow = group::Flex::default().row();
{
frame::Frame::default()
.with_label("Password:")
.with_align(enums::Align::Inside | enums::Align::Right);
let password = input::Input::default();
prow.fixed(&password, 180);
prow.end();
}
let pad = frame::Frame::default();
let mut brow = group::Flex::default().row();
{
frame::Frame::default();
let reg = create_button("Register");
let login = create_button("Login");
brow.fixed(®, 80);
brow.fixed(&login, 80);
brow.end();
}
let b = frame::Frame::default();
frame::Frame::default();
parent.fixed(&w, 60);
parent.fixed(&urow, 30);
parent.fixed(&prow, 30);
parent.fixed(&pad, 1);
parent.fixed(&brow, 30);
parent.fixed(&b, 30);
}
fn middle_panel(parent: &mut group::Flex) {
frame::Frame::default();
let mut frame = frame::Frame::default().with_label("Image");
frame.set_frame(enums::FrameType::BorderBox);
frame.set_color(enums::Color::from_rgb(0, 200, 0));
let spacer = frame::Frame::default();
let mut bp = group::Flex::default().column();
buttons_panel(&mut bp);
bp.end();
frame::Frame::default();
parent.fixed(&frame, 200);
parent.fixed(&spacer, 10);
parent.fixed(&bp, 300);
}
fn main_panel(parent: &mut group::Flex) {
frame::Frame::default();
let mut mp = group::Flex::default().row();
middle_panel(&mut mp);
mp.end();
frame::Frame::default();
parent.fixed(&mp, 200);
}
fn create_button(caption: &str) -> button::Button {
let mut btn = button::Button::default().with_label(caption);
btn.set_color(enums::Color::from_rgb(225, 225, 225));
btn
}More examples
17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115
fn main() {
let app = app::App::default();
// Act as the application in the taskbar (scroll to event handling)
let mut dock_win = window::Window::default()
.with_size(1, 1) // So we can place it at the center of the screen (needs a size >0 to be centered)
.with_label("TestApplication")
.center_screen();
dock_win.size_range(0, 0, 0, 0);
dock_win.make_resizable(false);
dock_win.show();
dock_win.end();
let mut win = window::Window::default()
.with_size(900, 500)
.with_label("TestApplication")
.center_screen();
win.set_color(enums::Color::from_rgb(26, 25, 55));
let mut but = button::Button::default()
.with_label("Button")
.with_size(80, 80)
.center_of_parent();
but.set_frame(enums::FrameType::OFlatFrame);
but.set_color(enums::Color::Cyan);
but.clear_visible_focus();
but.set_callback(|_| println!("Clicked"));
win.show();
win.end();
let win_shape = prep_shape(win.w(), win.h());
// Called after showing window
win.set_shape(Some(win_shape));
win.handle({
let mut x = 0;
let mut y = 0;
let mut dock_win = dock_win.clone();
move |wself, event| match event {
enums::Event::Push => {
let coords = app::event_coords();
x = coords.0;
y = coords.1;
true
}
enums::Event::Drag => {
wself.set_pos(app::event_x_root() - x, app::event_y_root() - y);
// Changing dock window position so it's close enough to the center of the application (not "visible" to user)
dock_win.set_pos(wself.x() + (wself.w() / 2), wself.y() + (wself.w() / 2));
true
}
enums::Event::Close => {
app.quit();
true
}
enums::Event::Hide => {
app.quit();
true
}
_ => false,
}
});
// Make main window appear when "opened" via Alt+Tab or Taskbar
dock_win.handle({
let mut win = win.clone();
move |_wself, event| match event {
enums::Event::Focus => {
let win_shape = prep_shape(win.w(), win.h());
win.show();
win.set_shape(Some(win_shape));
true
}
enums::Event::Hide => {
win.hide();
true
}
enums::Event::Close => {
app.quit();
true
}
_ => false,
}
});
app.run().unwrap();
}506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657
fn mb_test4_cb(_choice: &mut fltk::menu::Choice, term: &mut Terminal) {
// Test the Utf8Char primitive
let uc = Utf8Char::new(b'Q');
let uc1 = uc.text_utf8();
assert_eq!(&uc1, &[b'Q']);
assert_eq!(&uc.attrib(), &Attrib::Normal);
assert_eq!(
&uc.charflags(),
&(CharFlags::FG_XTERM | CharFlags::BG_XTERM)
);
assert_eq!(&uc.fgcolor(), &Color::XtermWhite);
assert_eq!(&uc.bgcolor(), &Color::TransparentBg);
let ring_rows = term.ring_rows();
// println!();
// dbg!(term.disp_srow(), term.disp_erow(), term.disp_rows(), term.ring_cols(), term.ring_srow(), term.ring_erow() );
// dbg!(term.hist_srow(), term.hist_erow(), term.hist_rows(), ring_rows );
// dbg!(term.offset(), term.hist_use_srow(), term.hist_use() );
term.take_focus().unwrap();
term.clear_history();
assert_eq!(term.history_use(), 0);
// Subtract row numbers, modulo `rows`
fn row_diff(rows: i32, a: i32, b: i32) -> i32 {
match a - b {
n if n < 0 => n + rows,
n => n
}
}
// disp_srow is always 1 greater than hist_erow, modulo (ring_rows+1)
assert_eq!(row_diff(ring_rows, term.disp_srow(), term.hist_erow()), 1);
assert!(term.disp_srow() >= 0);
assert!(term.disp_erow() >= 0);
assert!(term.hist_srow() >= 0);
assert!(term.hist_erow() >= 0);
assert!(term.offset() >= 0);
assert!(term.disp_srow() <= ring_rows);
assert!(term.disp_erow() <= ring_rows);
assert!(term.hist_srow() <= ring_rows);
assert!(term.hist_erow() <= ring_rows);
assert!(term.offset() <= ring_rows);
assert_eq!(term.ring_srow(), 0);
assert_eq!(term.ring_erow(), ring_rows - 1);
assert_eq!(
row_diff(ring_rows, term.disp_erow(), term.disp_srow()) + 1,
term.disp_rows()
);
assert_eq!(
row_diff(ring_rows, term.hist_erow(), term.hist_srow()) + 1,
term.hist_rows()
);
assert_eq!(term.ring_erow(), term.ring_rows() - 1);
assert_eq!(term.ring_srow(), 0);
// Check the different cols methods, which should all return the same answer
assert!(term.disp_cols() > 10);
assert_eq!(term.disp_cols(), term.ring_cols());
assert_eq!(term.disp_cols(), term.hist_cols());
// Redundant protected vs public methods:
assert_eq!(term.disp_cols(), term.display_columns());
assert_eq!(term.disp_rows(), term.display_rows());
/// Local function to read back all rows from the display into a long string.
/// Does not include scrollback history.
/// Trims trailing blanks on each line
fn read_disp(term: &Terminal) -> String {
let rows = term.display_rows();
let mut text: Vec<u8> = Vec::with_capacity((rows * 64) as usize);
for row in 0..rows {
let r = term.u8c_disp_row(row).trim();
// Iterate through a row, accumulating [u8]
for c in r.iter() {
// Note: Sometimes utf-8 length is > 1
text.extend_from_slice(c.text_utf8());
}
text.extend_from_slice(b"\n");
}
// Return the result as a string
std::str::from_utf8(&text).unwrap().to_string()
}
term.clear();
term.append("Top line ↑ (up-arrow)");
term.set_text_attrib(Attrib::Underline);
term.append(" ");
term.set_text_attrib(Attrib::Normal);
term.append(" \n");
let mut text_out = read_disp(term);
// Trim trailing empty lines
text_out = text_out.trim_end_matches(&"\n\n").to_string();
// The two plain blanks at the end will be trimmed, the two underlined blanks will be retained.
assert_eq!(text_out, "Top line ↑ (up-arrow) \n");
let r = term.u8c_disp_row(0);
assert_eq!(r.col(0).text_utf8(), b"T");
assert_eq!(r.col(10).text_utf8(), b"\xe2\x86\x91"); // UTF-8 up-arrow
assert_eq!(r.col(24).text_utf8(), b" "); // First blank after test text, NOT trimmed
let r = term.u8c_disp_row(1);
assert_eq!(r.col(0).text_utf8(), b" "); // Second row starts with blanks
assert_eq!(r.col(1).text_utf8(), b" "); // Second row is full of blanks
// Clear the screen again, then append test text, then read it back and compare
let test_text =
"The wind was a torrent of darkness among the gusty trees.
The moon was a ghostly galleon tossed upon cloudy seas.
The road was a ribbon of moonlight over the purple moor,
And the highwayman came riding—
Riding—riding—
The highwayman came riding, up to the old inn-door.";
term.clear_history();
term.clear();
let bg_save = term.text_bg_color();
let fg_save = term.text_fg_color();
term.set_text_bg_color(Color::DarkBlue); // Set spooky colors
term.set_text_fg_color(Color::from_rgb(0x40, 0x40, 0xff));
term.append(test_text);
term.set_text_bg_color(bg_save);
term.set_text_fg_color(fg_save);
let mut text_out = read_disp(term);
// Trim trailing empty lines
text_out = text_out.trim_end_matches(&"\n\n").to_string();
assert_eq!(test_text, text_out);
assert_eq!(row_diff(ring_rows, term.disp_srow(), term.hist_erow()), 1);
assert_eq!(term.ring_srow(), 0);
assert_eq!(term.ring_erow(), ring_rows - 1);
assert_eq!(
row_diff(ring_rows, term.disp_erow(), term.disp_srow()) + 1,
term.disp_rows()
);
assert_eq!(
row_diff(ring_rows, term.hist_erow(), term.hist_srow()) + 1,
term.hist_rows()
);
term.append(&format!(
"\n\nScreen has {} rows of {} columns.\n",
term.disp_rows(),
term.disp_cols()
));
term.set_text_attrib(Attrib::Italic);
term.append("Done!");
term.set_text_attrib(Attrib::Normal);
}sourcepub const fn from_rgba(r: u8, g: u8, b: u8, a: u8) -> Color
Available on crate feature enable-glwindow only.
pub const fn from_rgba(r: u8, g: u8, b: u8, a: u8) -> Color
enable-glwindow only.Returns a color from RGB
sourcepub fn from_rgba_tuple(tup: (u8, u8, u8, u8)) -> Color
pub fn from_rgba_tuple(tup: (u8, u8, u8, u8)) -> Color
Create color from RGBA using alpha compositing. Works for non-group types.
sourcepub const fn from_u32(val: u32) -> Color
pub const fn from_u32(val: u32) -> Color
Returns a color from hex or decimal
Examples found in repository?
More examples
64 65 66 67 68 69 70 71 72 73 74 75 76 77 78
fn main() {
let app = app::App::default();
app::background(255, 255, 255);
let mut win = window::Window::default().with_size(400, 300);
let mut dial = MyDial::new(100, 100, 200, 200, "CPU Load %");
dial.set_label_size(22);
dial.set_label_color(Color::from_u32(0x797979));
win.end();
win.show();
// get the cpu load value from somewhere, then call dial.set_value() in a callback or event loop
dial.set_value(10);
app.run().unwrap();
}63 64 65 66 67 68 69 70 71 72 73 74 75 76
fn draw_data(txt: &str, x: i32, y: i32, w: i32, h: i32, selected: bool) {
draw::push_clip(x, y, w, h);
if selected {
draw::set_draw_color(enums::Color::from_u32(0x00D3_D3D3));
} else {
draw::set_draw_color(enums::Color::White);
}
draw::draw_rectf(x, y, w, h);
draw::set_draw_color(enums::Color::Gray0);
draw::set_font(enums::Font::Helvetica, 14);
draw::draw_text2(txt, x, y, w, h, enums::Align::Center);
draw::draw_rect(x, y, w, h);
draw::pop_clip();
}42 43 44 45 46 47 48 49 50 51 52 53 54 55 56
pub fn new(buf: text::TextBuffer) -> Self {
let mut editor = text::TextEditor::new(5, 35, 790, 560, "");
editor.set_buffer(Some(buf));
#[cfg(target_os = "macos")]
editor.resize(5, 5, 790, 590);
editor.set_scrollbar_size(15);
editor.set_text_font(Font::Courier);
editor.set_linenumber_width(32);
editor.set_linenumber_fgcolor(Color::from_u32(0x008b_8386));
editor.set_trigger(CallbackTrigger::Changed);
Self { editor }
}129 130 131 132 133 134 135 136 137 138 139 140 141 142
fn draw_data(txt: &str, x: i32, y: i32, w: i32, h: i32, selected: bool) {
draw::push_clip(x, y, w, h);
if selected {
draw::set_draw_color(enums::Color::from_u32(0x00D3_D3D3));
} else {
draw::set_draw_color(enums::Color::White);
}
draw::draw_rectf(x, y, w, h);
draw::set_draw_color(enums::Color::Gray0);
draw::set_font(enums::Font::Helvetica, 14);
draw::draw_text2(txt, x, y, w, h, enums::Align::Center);
draw::draw_rect(x, y, w, h);
draw::pop_clip();
}9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27
pub fn new(w: i32, h: i32) -> MyButton {
let mut grp = group::Group::new(0, 0, w, h, None);
grp.set_frame(enums::FrameType::RFlatBox);
grp.set_color(enums::Color::from_u32(0x01579b));
grp.set_align(enums::Align::Center);
let mut btn = button::Button::new(grp.x() + 420, grp.y() + 35, 30, 25, "@1+");
btn.set_frame(enums::FrameType::OFlatFrame);
btn.set_color(enums::Color::from_u32(0xf49da9));
btn.set_callback(move |b| b.parent().unwrap().hide());
grp.end();
grp.handle(|g, ev| match ev {
enums::Event::Push => {
g.do_callback();
true
}
_ => false,
});
MyButton { grp }
}sourcepub const fn from_hex(val: u32) -> Color
pub const fn from_hex(val: u32) -> Color
Returns a color from hex or decimal
Examples found in repository?
More examples
37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280
pub fn new(title: &'static str) -> MyButton {
let mut b = Button::new(0, 0, 100, 0, title);
b.set_label_size(24);
b.set_frame(FrameType::FlatBox);
match title {
"CE" => {
b.set_color(Color::from_hex(0xd50000));
b.set_shortcut(Shortcut::None | Key::Delete);
}
"x" | "/" | "+" | "-" | "=" | "C" | "@<-" => {
b.set_color(Color::from_hex(0xffee58));
b.set_label_color(Color::Black);
let shortcut = if title == "x" {
'*'
} else {
title.chars().next().unwrap()
};
b.set_shortcut(Shortcut::None | shortcut);
if shortcut == '@' {
b.set_shortcut(Shortcut::None | Key::BackSpace);
}
if shortcut == '=' {
b.set_shortcut(Shortcut::None | Key::Enter);
}
}
_ => {
if title == "0" {
b.resize(0, 0, 100 * 2, 0);
}
b.set_label_color(Color::White);
b.set_selection_color(Color::from_hex(0x1b1b1b));
b.set_shortcut(Shortcut::None | title.chars().next().unwrap());
b.handle(move |b, ev| match ev {
Event::Enter => {
b.set_color(Color::from_hex(0x2b2b2b));
b.redraw();
true
}
Event::Leave => {
b.set_color(Color::from_hex(0x424242));
b.redraw();
true
}
_ => false,
});
}
}
Self { b }
}
}
impl Deref for MyButton {
type Target = Button;
fn deref(&self) -> &Self::Target {
&self.b
}
}
impl DerefMut for MyButton {
fn deref_mut(&mut self) -> &mut Self::Target {
&mut self.b
}
}
fn main() {
let app = app::App::default();
app::set_visible_focus(false);
app::background(0x42, 0x42, 0x42);
let win_w = 400;
let win_h = 500;
let but_row = 160;
let mut operation = Ops::None;
let mut txt = String::from("0");
let mut old_val = String::from("0");
let mut new_val: String;
let mut wind = Window::default()
.with_label("FLTK Calc")
.with_size(win_w, win_h)
.center_screen();
let mut out = Frame::new(0, 0, win_w, 160, "").with_align(Align::Right | Align::Inside);
out.set_color(Color::from_hex(0x1b1b1b));
out.set_frame(FrameType::FlatBox);
out.set_label_color(Color::White);
out.set_label_size(36);
out.set_label("0");
let vpack = Pack::new(0, but_row, win_w, win_h - 170, "");
let mut hpack = Pack::new(0, 0, win_w, 68, "");
let but_ce = MyButton::new("CE");
let but_c = MyButton::new("C");
let but_back = MyButton::new("@<-");
let but_div = MyButton::new("/");
hpack.end();
hpack.set_type(PackType::Horizontal);
let mut hpack = Pack::new(0, 0, win_w, 68, "");
let mut but7 = MyButton::new("7");
let mut but8 = MyButton::new("8");
let mut but9 = MyButton::new("9");
let but_mul = MyButton::new("x");
hpack.end();
hpack.set_type(PackType::Horizontal);
let mut hpack = Pack::new(0, 0, win_w, 68, "");
let mut but4 = MyButton::new("4");
let mut but5 = MyButton::new("5");
let mut but6 = MyButton::new("6");
let but_sub = MyButton::new("-");
hpack.end();
hpack.set_type(PackType::Horizontal);
let mut hpack = Pack::new(0, 0, win_w, 68, "");
let mut but1 = MyButton::new("1");
let mut but2 = MyButton::new("2");
let mut but3 = MyButton::new("3");
let but_add = MyButton::new("+");
hpack.end();
hpack.set_type(PackType::Horizontal);
let mut hpack = Pack::new(0, 0, win_w, 68, "");
let mut but_dot = MyButton::new(".");
let mut but0 = MyButton::new("0");
let but_eq = MyButton::new("=");
hpack.end();
hpack.set_type(PackType::Horizontal);
vpack.end();
wind.make_resizable(false);
wind.end();
wind.show();
app::set_focus(&*but1);
app::get_system_colors();
let but_vec = vec![
&mut but1, &mut but2, &mut but3, &mut but4, &mut but5, &mut but6, &mut but7, &mut but8,
&mut but9, &mut but0,
];
let but_op_vec = vec![
but_add, but_sub, but_mul, but_div, but_c, but_ce, but_back, but_eq,
];
let (s, r) = app::channel::<Message>();
for but in but_vec {
let label = but.label();
but.emit(s, Message::Number(label.parse().unwrap()));
}
for mut but in but_op_vec {
let op = match but.label().as_str() {
"+" => Ops::Add,
"-" => Ops::Sub,
"x" => Ops::Mul,
"/" => Ops::Div,
"=" => Ops::Eq,
"CE" => Ops::CE,
"C" => Ops::C,
"@<-" => Ops::Back,
_ => Ops::None,
};
but.emit(s, Message::Op(op));
}
but_dot.emit(s, Message::Dot);
while app.wait() {
if let Some(val) = r.recv() {
match val {
Message::Number(num) => {
if out.label() == "0" {
txt.clear();
}
txt.push_str(&num.to_string());
out.set_label(txt.as_str());
}
Message::Dot => {
if operation == Ops::Eq {
txt.clear();
operation = Ops::None;
out.set_label("0.");
txt.push_str("0.");
}
if !txt.contains('.') {
txt.push('.');
out.set_label(txt.as_str());
}
}
Message::Op(op) => match op {
Ops::Add | Ops::Sub | Ops::Div | Ops::Mul => {
old_val.clear();
old_val.push_str(&out.label());
operation = op;
out.set_label("0");
}
Ops::Back => {
let val = out.label();
txt.pop();
if val.len() > 1 {
out.set_label(txt.as_str());
} else {
out.set_label("0");
}
}
Ops::CE => {
txt.clear();
old_val.clear();
txt.push('0');
out.set_label(txt.as_str());
}
Ops::C => {
txt.clear();
txt.push('0');
out.set_label(txt.as_str());
}
Ops::Eq => {
new_val = out.label();
let old: f64 = old_val.parse().unwrap();
let new: f64 = new_val.parse().unwrap();
let val = match operation {
Ops::Div => old / new,
Ops::Mul => old * new,
Ops::Add => old + new,
Ops::Sub => old - new,
_ => new,
};
operation = Ops::None;
txt = String::from("0");
out.set_label(&val.to_string());
}
_ => (),
},
}
}
}
}17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371
fn main() {
let app = fltk::app::App::default();
// Set panic handler for main thread (will become UI thread)
std::panic::set_hook(Box::new({
|e| {
eprintln!("!!!!PANIC!!!!{:#?}", e);
error_box(e.to_string()); // Only works from the UI thread
std::process::exit(2);
}
}));
let mut main_win = Window::new(
2285,
180,
WIN_WIDTH,
WIN_HEIGHT,
"FLTK/Terminal Rust wrapper test",
);
main_win.set_type(WindowType::Double);
main_win.make_resizable(true);
let mut menu_bar = MenuBar::new(0, 0, WIN_WIDTH, 30, None);
let mut term = Terminal::new(0, 30, WIN_WIDTH, WIN_HEIGHT - 30, None);
term.set_label("term");
main_win.resizable(&term);
term.set_label_type(LabelType::None);
let idx = menu_bar.add_choice("Test&1");
menu_bar.at(idx).unwrap().set_callback({
let mut term1 = term.clone();
move |c| mb_test1_cb(c, &mut term1)
});
menu_bar
.at(idx)
.unwrap()
.set_shortcut(unsafe { std::mem::transmute(0x80031) }); // Alt-1
let idx = menu_bar.add_choice("Test&2");
menu_bar.at(idx).unwrap().set_callback({
let mut term1 = term.clone();
move |c| mb_test2_cb(c, &mut term1)
});
menu_bar
.at(idx)
.unwrap()
.set_shortcut(unsafe { std::mem::transmute(0x80032) }); // Alt-2
let idx = menu_bar.add_choice("Test&3");
menu_bar.at(idx).unwrap().set_callback({
let mut term1 = term.clone();
move |c| mb_test3_cb(c, &mut term1)
});
menu_bar
.at(idx)
.unwrap()
.set_shortcut(unsafe { std::mem::transmute(0x80033) }); // Alt-3
let idx = menu_bar.add_choice("Test&4");
menu_bar.at(idx).unwrap().set_callback({
let mut term1 = term.clone();
move |c| mb_test4_cb(c, &mut term1)
});
menu_bar
.at(idx)
.unwrap()
.set_shortcut(unsafe { std::mem::transmute(0x80034) }); // Alt-4
menu_bar.end();
main_win.end();
main_win.show();
// Worker thread that drives the startup tests
let _worker_thread: std::thread::JoinHandle<_> = std::thread::spawn({
let mut term = term.clone();
move || {
println!("Startup tests\n");
term.append("Startup tests\n\n");
// Testing ansi() and set_ansi() methods
assert!(term.ansi(), "Default ANSI mode should be ON at startup");
term.append("ANSI mode is \x1b[4mON\x1b[0m\n");
term.set_ansi(false);
assert!(!term.ansi());
term.append("ANSI mode is \x1b[4mOFF\x1b[0m\n");
// append() method is already being used/tested. Test the u8, ascii, and utf8 variants
term.append_u8(b"Appending u8 array\n");
term.append_ascii("Appending ASCII array ↑ (up-arrow is dropped)\n");
// Test show_unknown() as incidental part of testing append methods
term.set_show_unknown(true);
assert!(term.show_unknown());
term.append_ascii(
"Appending ASCII array with show_unknown() ↑ (up-arrow is three unknown bytes)\n",
);
term.set_show_unknown(false);
assert!(!term.show_unknown());
term.append_utf8("Appending UTF8 array ↑ (up-arrow is visible)\n");
term.append_utf8_u8(b"Appending UTF8 array as u8 \xe2\x86\x91 (up-arrow is visible)\n");
let r = term.cursor_row();
assert_eq!(term.cursor_col(), 0);
term.append(&format!("Testing cursor row/col {r}"));
assert_eq!(term.cursor_col(), 24);
assert_eq!(term.cursor_row(), r);
// Test cursor color methods
assert_eq!(
term.cursor_bg_color(),
Color::XtermGreen,
"Default cursor bg at startup"
);
assert_eq!(
term.cursor_fg_color(),
Color::from_hex(0xff_ff_f0),
"Default cursor fg at startup"
);
term.set_cursor_bg_color(Color::Red);
assert_eq!(term.cursor_bg_color(), Color::Red);
assert_eq!(term.cursor_fg_color(), Color::from_hex(0xff_ff_f0));
term.set_cursor_fg_color(Color::Blue);
assert_eq!(term.cursor_bg_color(), Color::Red);
assert_eq!(term.cursor_fg_color(), Color::Blue);
term.set_cursor_bg_color(Color::XtermGreen); // Restore the defaults
term.set_cursor_fg_color(Color::from_hex(0xff_ff_f0));
assert_eq!(term.cursor_bg_color(), Color::XtermGreen);
assert_eq!(term.cursor_fg_color(), Color::from_hex(0xff_ff_f0));
// The default display_rows() will derive from the window size
let dr = term.display_rows();
assert!(dr > 20, "Default display_rows at startup");
term.set_display_rows(60);
assert_eq!(term.display_rows(), 60);
term.set_display_rows(dr); // Set back to default
assert_eq!(term.display_rows(), dr);
// The default display_columns() will derive from the window size
let dc = term.display_columns();
assert!(dc > 80, "Default display_rows at startup");
term.set_display_columns(200);
assert_eq!(term.display_columns(), 200);
term.append("\n 1 2 3 4 5 6 7 8 9");
term.append("\n123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890");
term.append("[This text should be truncated by display_columns() call below.]\n"); // We shouldn't see this on screen
term.set_display_columns(90);
assert_eq!(term.display_columns(), 90);
term.set_display_columns(dc); // Set back to default
assert_eq!(term.display_columns(), dc);
let hl = term.history_lines();
assert_eq!(hl, 100, "Default history_lines at startup");
term.set_history_lines(60);
assert_eq!(term.history_lines(), 60);
term.set_history_lines(hl); // Set back to default
assert_eq!(term.history_lines(), hl);
// Is history_rows() an alias for history_lines()?
assert_eq!(term.history_rows(), 100, "Default history_rows at startup");
term.set_history_rows(50);
assert_eq!(term.history_rows(), 50);
term.set_history_lines(100); // Set back to default
assert_eq!(term.history_lines(), 100);
let hu = term.history_use();
term.append(&format!(
"history_use = {hu} (it's not clear what this means)\n"
));
// assert_eq!(term.history_use(), hu+1);
term.append(&format!(
"margins = b:{} l:{} r:{} t{}\n",
term.margin_bottom(),
term.margin_left(),
term.margin_right(),
term.margin_top()
));
assert_eq!(term.margin_bottom(), 3);
assert_eq!(term.margin_left(), 3);
assert_eq!(term.margin_right(), 3);
assert_eq!(term.margin_top(), 3);
term.set_margin_bottom(5);
term.set_margin_left(10);
term.set_margin_right(15);
term.set_margin_top(20);
assert_eq!(term.margin_bottom(), 5);
assert_eq!(term.margin_left(), 10);
assert_eq!(term.margin_right(), 15);
assert_eq!(term.margin_top(), 20);
term.append("Single character: '");
term.print_char('X');
term.append("', single UTF-8 character: '");
term.print_char_utf8('↑');
term.append("'\n");
let rr = term.redraw_rate();
assert_eq!(rr, 0.1, "Default redraw rate at startup");
term.append(&format!("Redraw rate {rr}\n"));
term.set_redraw_rate(1.0);
assert_eq!(term.redraw_rate(), 1.0);
term.set_redraw_rate(rr);
assert_eq!(term.redraw_rate(), rr);
let rs = term.redraw_style();
term.append(&format!("Redraw style {rs:?}\n"));
assert_eq!(
rs,
RedrawStyle::RateLimited,
"Default redraw style at startup"
);
term.set_redraw_style(RedrawStyle::NoRedraw);
assert_eq!(term.redraw_style(), RedrawStyle::NoRedraw);
term.set_redraw_style(rs);
assert_eq!(term.redraw_style(), rs);
// Sanity checks: enum values are implicitly assigned in the C++ code so could change unexpectedly
assert_eq!(
RedrawStyle::NoRedraw.bits(),
0x0000,
"RedrawStyle enum values have been reassigned"
);
assert_eq!(
RedrawStyle::RateLimited.bits(),
0x0001,
"RedrawStyle enum values have been reassigned"
);
assert_eq!(
RedrawStyle::PerWrite.bits(),
0x0002,
"RedrawStyle enum values have been reassigned"
);
term.append(&format!(
"Scrollbar actual size {}\n",
term.scrollbar_actual_size()
));
assert_eq!(term.scrollbar_actual_size(), 16);
term.append(&format!("Scrollbar size {}\n", term.scrollbar_size()));
assert_eq!(
term.scrollbar_size(),
0,
"Default scrollbar size at startup"
);
term.set_scrollbar_size(40);
assert_eq!(term.scrollbar_size(), 40);
assert_eq!(term.scrollbar_actual_size(), 40);
term.append(&format!(
"Scrollbar actual size {}\n",
term.scrollbar_actual_size()
));
term.set_scrollbar_size(0); // Restore default
assert_eq!(term.scrollbar_size(), 0);
assert_eq!(term.scrollbar_actual_size(), 16);
let sfc = term.selection_fg_color();
let sbc = term.selection_bg_color();
assert_eq!(sfc, Color::Black);
assert_eq!(sbc, Color::White);
term.append(&format!("Selection colors: {sfc} {sbc}\n"));
term.set_selection_fg_color(Color::Green);
term.set_selection_bg_color(Color::DarkBlue);
assert_eq!(term.selection_fg_color(), Color::Green);
assert_eq!(term.selection_bg_color(), Color::DarkBlue);
term.set_selection_fg_color(sfc);
term.set_selection_bg_color(sbc);
assert_eq!(term.selection_fg_color(), Color::Black);
assert_eq!(term.selection_bg_color(), Color::White);
let tfcd = term.text_fg_color_default();
let tbcd = term.text_bg_color_default();
assert_eq!(tfcd, Color::XtermWhite);
assert_eq!(tbcd, Color::TransparentBg);
term.append(&format!("Default text colors: {sfc} {sbc}\n"));
term.set_text_fg_color_default(Color::Green);
term.set_text_bg_color_default(Color::DarkBlue);
assert_eq!(term.text_fg_color_default(), Color::Green);
assert_eq!(term.text_bg_color_default(), Color::DarkBlue);
term.set_text_fg_color_default(tfcd);
term.set_text_bg_color_default(tbcd);
assert_eq!(term.text_fg_color_default(), Color::XtermWhite);
assert_eq!(term.text_bg_color_default(), Color::TransparentBg);
let tfc = term.text_fg_color();
let tbc = term.text_bg_color();
assert_eq!(tfc, Color::XtermWhite);
assert_eq!(tbc, Color::TransparentBg);
term.append(&format!("Text colors: {sfc} {sbc}\n"));
term.set_text_fg_color(Color::Green);
term.set_text_bg_color(Color::DarkBlue);
assert_eq!(term.text_fg_color(), Color::Green);
assert_eq!(term.text_bg_color(), Color::DarkBlue);
term.set_text_fg_color(tfc);
term.set_text_bg_color(tbc);
assert_eq!(term.text_fg_color(), Color::XtermWhite);
assert_eq!(term.text_bg_color(), Color::TransparentBg);
let tf = term.text_font();
term.append(&format!("Text font: {tf:?}\n"));
assert_eq!(tf, Font::Courier);
term.set_text_font(Font::Screen);
assert_eq!(term.text_font(), Font::Screen);
term.set_text_font(tf);
assert_eq!(term.text_font(), Font::Courier);
let ts = term.text_size();
term.append(&format!("Text size: {ts}\n"));
assert_eq!(ts, 14);
term.set_text_size(30);
assert_eq!(term.text_size(), 30);
term.set_text_size(ts);
assert_eq!(term.text_size(), ts);
// Keyboard handler
term.handle({
let mut term = term.clone();
move |_kc, e| {
match e {
fltk::enums::Event::KeyDown
if fltk::app::event_key() == fltk::enums::Key::Escape =>
{
// let FLTK handle ESC
false
}
fltk::enums::Event::KeyDown
if fltk::app::event_length() == 1 && !fltk::app::is_event_alt() =>
{
// We handle normal printable keystroke
let k = fltk::app::event_text();
term.take_focus().unwrap();
term.append(&k);
true
}
// fltk docs say that keyboard handler should always claim Focus and Unfocus events
fltk::enums::Event::Focus | fltk::enums::Event::Unfocus => true,
_ => false, // Let FLTK handle everything else
}
}
});
term.set_text_attrib(Attrib::Inverse | Attrib::Italic);
term.append("\nStartup tests complete. Keyboard is live.\n");
term.set_text_attrib(Attrib::Normal);
term.redraw();
}
});
app.run().unwrap();
}sourcepub fn from_hex_str(col: &str) -> Result<Color, FltkError>
pub fn from_hex_str(col: &str) -> Result<Color, FltkError>
Return a Color from a hex color format (#xxxxxx)
sourcepub fn to_hex_str(&self) -> String
pub fn to_hex_str(&self) -> String
Returns the color in hex string format
sourcepub fn by_index(idx: u8) -> Color
pub fn by_index(idx: u8) -> Color
Returns a color by index of RGBI
Examples found in repository?
3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71
fn main() {
let app = app::App::default();
let mut window = window::Window::default().with_size(300, 300);
window.set_frame(FrameType::NoBox);
window.make_resizable(true);
let dx = 20;
let dy = dx; // border width of resizable() - see below
let tile = group::Tile::default_fill();
// create the symmetrical resize box with dx and dy pixels distance, resp.
// from the borders of the Fl_Tile widget before all other children
let r = frame::Frame::new(
tile.x() + dx,
tile.y() + dy,
tile.w() - 2 * dx,
tile.h() - 2 * dy,
None,
);
tile.resizable(&r);
let mut box0 = frame::Frame::new(0, 0, 150, 150, "0");
box0.set_frame(FrameType::DownBox);
box0.set_color(Color::by_index(9));
box0.set_label_size(36);
box0.set_align(Align::Clip);
let mut w1 = window::Window::new(150, 0, 150, 150, "1");
w1.set_frame(FrameType::NoBox);
let mut box1 = frame::Frame::new(0, 0, 150, 150, "1\nThis is a child window");
box1.set_frame(FrameType::DownBox);
box1.set_color(Color::by_index(19));
box1.set_label_size(18);
box1.set_align(Align::Clip | Align::Inside | Align::Wrap);
w1.resizable(&box1);
w1.end();
let mut box2a = frame::Frame::new(0, 150, 70, 150, "2a");
box2a.set_frame(FrameType::DownBox);
box2a.set_color(Color::by_index(12));
box2a.set_label_size(36);
box2a.set_align(Align::Clip);
let mut box2b = frame::Frame::new(70, 150, 80, 150, "2b");
box2b.set_frame(FrameType::DownBox);
box2b.set_color(Color::by_index(13));
box2b.set_label_size(36);
box2b.set_align(Align::Clip);
let mut box3a = frame::Frame::new(150, 150, 150, 70, "3a");
box3a.set_frame(FrameType::DownBox);
box3a.set_color(Color::by_index(12));
box3a.set_label_size(36);
box3a.set_align(Align::Clip);
let mut box3b = frame::Frame::new(150, 150 + 70, 150, 80, "3b");
box3b.set_frame(FrameType::DownBox);
box3b.set_color(Color::by_index(13));
box3b.set_label_size(36);
box3b.set_align(Align::Clip);
tile.end();
window.end();
w1.show();
window.show();
app.run().unwrap();
}sourcepub fn inactive(&self) -> Color
pub fn inactive(&self) -> Color
Returns an inactive form of the color
Examples found in repository?
5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91
fn create_vertical_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Vertical");
frame.draw(move |f| {
let imax = f.h();
let d = if imax > 0 { imax } else { 1 };
for i in 0..=imax {
let w = 1.0 - i as f32 / d as f32;
set_draw_color(Color::inactive(&Color::color_average(col1, col2, w)));
draw_xyline(f.x(), f.y() + i, f.x() + f.w());
}
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}
fn create_horizontal_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Horizontal");
frame.draw(move |f| {
let imax = f.w();
let d = if imax > 0 { imax } else { 1 };
for i in 0..=imax {
let w = 1.0 - i as f32 / d as f32;
set_draw_color(Color::inactive(&Color::color_average(col1, col2, w)));
draw_yxline(f.x() + i, f.y(), f.y() + f.h());
}
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}
fn create_horizontal_svg_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Svg");
frame.draw(move |f| {
let (r1, g1, b1) = Color::inactive(&col1).to_rgb();
let (r2, g2, b2) = Color::inactive(&col2).to_rgb();
let svg = format!(
"<svg viewBox='0 0 {} {}'>
<defs>
<linearGradient id='grad1' x1='0%' y1='0%' x2='0%' y2='100%'>
<stop offset='0%' style='stop-color:rgb({},{},{});stop-opacity:1' />
<stop offset='100%' style='stop-color:rgb({},{},{});stop-opacity:1' />
</linearGradient>
</defs>
<rect width='100%' height='100%' fill='url(#grad1)' />
</svg>",
f.w(),
f.h() + 1,
r1,
g1,
b1,
r2,
g2,
b2
);
let mut image = image::SvgImage::from_data(&svg).unwrap();
image.draw(f.x(), f.y(), f.w(), f.h());
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}sourcepub fn gray_ramp(val: i32) -> Color
pub fn gray_ramp(val: i32) -> Color
Returns a gray color value from black (i == 0) to white (i == FL_NUM_GRAY - 1)
sourcepub fn color_average(c1: Color, c2: Color, weight: f32) -> Color
pub fn color_average(c1: Color, c2: Color, weight: f32) -> Color
Returns a gray color value from black (i == 0) to white (i == FL_NUM_GRAY - 1)
Examples found in repository?
5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114
fn create_vertical_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Vertical");
frame.draw(move |f| {
let imax = f.h();
let d = if imax > 0 { imax } else { 1 };
for i in 0..=imax {
let w = 1.0 - i as f32 / d as f32;
set_draw_color(Color::inactive(&Color::color_average(col1, col2, w)));
draw_xyline(f.x(), f.y() + i, f.x() + f.w());
}
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}
fn create_horizontal_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Horizontal");
frame.draw(move |f| {
let imax = f.w();
let d = if imax > 0 { imax } else { 1 };
for i in 0..=imax {
let w = 1.0 - i as f32 / d as f32;
set_draw_color(Color::inactive(&Color::color_average(col1, col2, w)));
draw_yxline(f.x() + i, f.y(), f.y() + f.h());
}
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}
fn create_horizontal_svg_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Svg");
frame.draw(move |f| {
let (r1, g1, b1) = Color::inactive(&col1).to_rgb();
let (r2, g2, b2) = Color::inactive(&col2).to_rgb();
let svg = format!(
"<svg viewBox='0 0 {} {}'>
<defs>
<linearGradient id='grad1' x1='0%' y1='0%' x2='0%' y2='100%'>
<stop offset='0%' style='stop-color:rgb({},{},{});stop-opacity:1' />
<stop offset='100%' style='stop-color:rgb({},{},{});stop-opacity:1' />
</linearGradient>
</defs>
<rect width='100%' height='100%' fill='url(#grad1)' />
</svg>",
f.w(),
f.h() + 1,
r1,
g1,
b1,
r2,
g2,
b2
);
let mut image = image::SvgImage::from_data(&svg).unwrap();
image.draw(f.x(), f.y(), f.w(), f.h());
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}
fn main() {
let a = app::App::default();
let mut win = window::Window::default().with_size(300, 300);
create_vertical_gradient_frame(0, 0, 100, 100, Color::Red, Color::Cyan);
create_horizontal_gradient_frame(100, 0, 100, 100, Color::Red, Color::Cyan);
create_horizontal_svg_gradient_frame(200, 0, 100, 100, Color::Red, Color::Cyan);
win.end();
win.draw(|w| {
// vertical gradient
let imax = w.w();
let d = if imax > 0 { imax } else { 1 };
for i in 0..=imax {
let v = 1.0 - i as f32 / d as f32;
set_draw_color(Color::color_average(Color::Red, Color::Blue, v));
draw_yxline(i, 0, w.h());
}
w.draw_children();
});
win.make_resizable(true);
win.show();
a.run().unwrap();
}sourcepub fn contrast(fg: Color, bg: Color) -> Color
pub fn contrast(fg: Color, bg: Color) -> Color
Returns a color that contrasts with the background color.
sourcepub fn gray_scale(g: u8) -> Color
pub fn gray_scale(g: u8) -> Color
Returns the color closest to the passed grayscale value
sourcepub fn rgb_color(r: u8, g: u8, b: u8) -> Color
pub fn rgb_color(r: u8, g: u8, b: u8) -> Color
Returns the color closest to the passed rgb value
sourcepub fn to_rgb(&self) -> (u8, u8, u8)
pub fn to_rgb(&self) -> (u8, u8, u8)
Get the RGB value of the color
Examples found in repository?
53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91
fn create_horizontal_svg_gradient_frame(
x: i32,
y: i32,
w: i32,
h: i32,
col1: Color,
col2: Color,
) -> frame::Frame {
let mut frame = frame::Frame::new(x, y, w, h, "Svg");
frame.draw(move |f| {
let (r1, g1, b1) = Color::inactive(&col1).to_rgb();
let (r2, g2, b2) = Color::inactive(&col2).to_rgb();
let svg = format!(
"<svg viewBox='0 0 {} {}'>
<defs>
<linearGradient id='grad1' x1='0%' y1='0%' x2='0%' y2='100%'>
<stop offset='0%' style='stop-color:rgb({},{},{});stop-opacity:1' />
<stop offset='100%' style='stop-color:rgb({},{},{});stop-opacity:1' />
</linearGradient>
</defs>
<rect width='100%' height='100%' fill='url(#grad1)' />
</svg>",
f.w(),
f.h() + 1,
r1,
g1,
b1,
r2,
g2,
b2
);
let mut image = image::SvgImage::from_data(&svg).unwrap();
image.draw(f.x(), f.y(), f.w(), f.h());
set_draw_color(Color::Black);
set_font(Font::Helvetica, app::font_size());
draw_text2(&f.label(), f.x(), f.y(), f.w(), f.h(), f.align());
});
frame
}More examples
6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43
pub fn new(radius: i32, mut image: image::RgbImage) -> Self {
let mut frame = frame::Frame::new(0, 0, radius * 2, radius * 2, None);
frame.set_frame(enums::FrameType::FlatBox);
frame.draw(move |f| {
image.scale(f.w(), f.h(), false, true);
image.draw(f.x(), f.y(), f.w(), f.h());
let color = f.color().to_rgb();
let s = format!(
"<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n
<svg width='{}' height='{}'>\n
<rect x='{}'
y='{}'
rx='{}'
ry='{}'
width='{}'
height='{}'
fill='none'
stroke='rgb({}, {}, {})'
stroke-width='{}' />\n
</svg>\n",
f.w(),
f.h(),
-f.w() / 2,
-f.w() / 2,
f.w(),
f.w(),
f.w() + f.w(),
f.h() + f.w(),
color.0,
color.1,
color.2,
f.w()
);
let mut s = image::SvgImage::from_data(&s).unwrap();
s.draw(f.x(), f.y(), f.w(), f.h());
});
Self
}125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363
fn main() {
let style = Rc::from(RefCell::from(Style::new()));
let app = App::default().with_scheme(Scheme::Gleam);
let mut wind = Window::default()
.with_size(500, 200)
.with_label("Highlight");
let mut vpack = Pack::new(4, 4, 492, 192, "");
vpack.set_spacing(4);
let mut text_editor = TextEditor::default().with_size(492, 163);
let mut hpack = Pack::new(4, 4, 492, 25, "").with_type(PackType::Horizontal);
hpack.set_spacing(8);
let mut font = Choice::default().with_size(130, 25);
let mut choice = Choice::default().with_size(130, 25);
let mut size = Spinner::default().with_size(60, 25);
let mut color = Choice::default().with_size(100, 25);
let mut btn_clear = Button::default().with_size(40, 25).with_label("X");
hpack.end();
vpack.end();
wind.end();
wind.show();
text_editor.wrap_mode(fltk::text::WrapMode::AtBounds, 0);
text_editor.set_buffer(TextBuffer::default());
font.add_choice("Courier|Helvetica|Times");
font.set_value(0);
font.set_tooltip("Font");
choice.add_choice("Normal|Underline|Strike");
choice.set_value(0);
size.set_value(18.0);
size.set_step(1.0);
size.set_range(12.0, 28.0);
size.set_tooltip("Size");
color.set_tooltip("Color");
color.add_choice("#000000|#ff0000|#00ff00|#0000ff|#ffff00|#00ffff");
color.set_value(0);
btn_clear.set_label_color(Color::Red);
btn_clear.set_tooltip("Clear style");
// set colors
for mut item in color.clone() {
if let Some(lbl) = item.label() {
item.set_label_color(Color::from_u32(
u32::from_str_radix(lbl.trim().strip_prefix('#').unwrap(), 16)
.ok()
.unwrap(),
));
}
}
let style_rc1 = Rc::clone(&style);
text_editor.buffer().unwrap().add_modify_callback({
let mut text_editor1 = text_editor.clone();
let font1 = font.clone();
let size1 = size.clone();
let color1 = color.clone();
let choice1 = choice.clone();
move |pos: i32, ins_items: i32, del_items: i32, _: i32, _: &str| {
let attr = if choice1.value() == 1 {
TextAttr::Underline
} else if choice1.value() == 2 {
TextAttr::StrikeThrough
} else {
TextAttr::None
};
if ins_items > 0 || del_items > 0 {
let mut style = style_rc1.borrow_mut();
let color = Color::from_u32(
u32::from_str_radix(
color1
.text(color1.value())
.unwrap()
.trim()
.strip_prefix('#')
.unwrap(),
16,
)
.ok()
.unwrap(),
);
style.apply_style(
Some(pos),
Some(ins_items),
Some(del_items),
None,
None,
Font::by_name(font1.text(font1.value()).unwrap().trim()),
size1.value() as i32,
color,
attr,
&mut text_editor1,
);
}
}
});
color.set_callback({
let size = size.clone();
let font = font.clone();
let choice = choice.clone();
let mut text_editor = text_editor.clone();
let style_rc1 = Rc::clone(&style);
move |color| {
let attr = match choice.value() {
0 => TextAttr::None,
1 => TextAttr::Underline,
2 => TextAttr::StrikeThrough,
_ => unreachable!(),
};
if let Some(buf) = text_editor.buffer() {
if let Some((s, e)) = buf.selection_position() {
let mut style = style_rc1.borrow_mut();
let color = Color::from_u32(
u32::from_str_radix(
color
.text(color.value())
.unwrap()
.trim()
.strip_prefix('#')
.unwrap(),
16,
)
.ok()
.unwrap(),
);
style.apply_style(
None,
None,
None,
Some(s),
Some(e),
Font::by_name(font.text(font.value()).unwrap().trim()),
size.value() as i32,
color,
attr,
&mut text_editor,
);
}
}
}
});
// get the style from the current cursor position
text_editor.handle({
let style_rc1 = Rc::clone(&style);
let mut font1 = font.clone();
let mut size1 = size.clone();
let mut color1 = color.clone();
move |te, e| match e {
Event::KeyUp | Event::Released => {
if let Some(buff) = te.style_buffer() {
let i = te.insert_position();
if let Some(t) = buff.text_range(i, i + 1) {
if !t.is_empty() {
let style = style_rc1.borrow_mut();
if let Some(i) = t.chars().next().map(|c| (c as usize - 65)) {
if let Some(style) = style.style_table.get(i) {
if let Some(mn) = font1.find_item(&format!("{:?}", style.font))
{
font1.set_item(&mn);
}
size1.set_value(style.size as f64);
let (r, g, b) = style.color.to_rgb();
if let Some(mn) =
color1.find_item(format!("{r:02x}{g:02x}{b:02x}").as_str())
{
color1.set_item(&mn);
}
}
}
}
}
}
true
}
_ => false,
}
});
choice.set_callback({
let mut color1 = color.clone();
move |_| color1.do_callback()
});
font.set_callback({
let mut color1 = color.clone();
move |_| color1.do_callback()
});
size.set_callback({
let mut color1 = color.clone();
move |_| color1.do_callback()
});
// clear style of the current selection or, if no text is selected, clear all text style
btn_clear.set_callback({
let style_rc1 = Rc::clone(&style);
let text_editor1 = text_editor.clone();
move |_| {
match text_editor1.buffer().unwrap().selection_position() {
Some((_, _)) => {
font.set_value(0);
size.set_value(18.0);
color.set_value(0);
choice.set_value(0);
color.do_callback();
}
None => {
font.set_value(0);
size.set_value(18.0);
color.set_value(0);
style_rc1.borrow_mut().apply_style(
None,
None,
None,
Some(0),
Some(text_editor1.buffer().unwrap().length()),
Font::Courier,
16,
Color::Black,
TextAttr::None,
&mut text_editor,
);
}
};
}
});
app.run().unwrap();
}Trait Implementations
sourceimpl Ord for Color
impl Ord for Color
sourceimpl PartialEq for Color
impl PartialEq for Color
sourceimpl PartialOrd for Color
impl PartialOrd for Color
1.0.0 · sourcefn le(&self, other: &Rhs) -> bool
fn le(&self, other: &Rhs) -> bool
self and other) and is used by the <=
operator. Read more